
package com.dft.onyx.plugin;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.NonNull;

import com.dft.onyxcamera.config.Onyx;
import com.dft.onyxcamera.config.OnyxConfiguration;
import com.dft.onyxcamera.config.OnyxConfigurationBuilder;
import com.dft.onyxcamera.config.OnyxError;
import com.dft.onyxcamera.config.OnyxResult;
import com.dft.onyxcamera.ui.reticles.Reticle;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * OnyxPlugin
 */
@SuppressLint("LogNotTimber")
public class OnyxCallbackHandler implements MethodCallHandler {
    public static Onyx configuredOnyx;
    public static Activity OnyxActivity;

    public OnyxCallbackHandler() {
        super();
    }

    // The fingerprint template valid when setReturnFingerprintTemplate(true) on
    // config
    // private ArrayList<FingerprintTemplate> fingerprintTemplates;
    /// The function that's triggered by the onyx success event.
    private final OnyxConfiguration.SuccessCallback successCallback = new OnyxConfiguration.SuccessCallback() {
        @Override
        public void onSuccess(OnyxResult onyxResult) {
            Map<String, Object> args = OnyxCallbackHelpers.getOnyxResultsMap(onyxResult);
            if (OnyxActivity != null) {
                OnyxActivity.finish();
            }
            OnyxPlugin.getActivity().runOnUiThread(() -> OnyxPlugin.channel.invokeMethod("onyx_success", args));
        }
    };

    /* ONYX-RESULT */

    // The processed fingerprint Bitmap images indexed 0-3 for each finger , 0-3
    // print indices
    // private ArrayList<Bitmap> processedFingerprintImages;

    // Wave-Length Scalar Quantization for grayscale prints
    // The WSQ data as a byte array
    // private ArrayList<byte[]> wsqData;
    /// The function that's triggered by the onyx error event.
    private final OnyxConfiguration.ErrorCallback errorCallback = new OnyxConfiguration.ErrorCallback() {
        @Override
        public void onError(OnyxError onyxError) {
            Log.i("OnyxPlugin", onyxError.getErrorMessage());
            Map<String, Object> args = new HashMap<>();
            args.put("errorMessage", onyxError.getErrorMessage());
            if (OnyxActivity != null) {
                OnyxActivity.finish();
            }
            OnyxPlugin.getActivity().runOnUiThread(() -> OnyxPlugin.channel.invokeMethod("onyx_error", args));
        }
    };
    private final OnyxConfiguration.OnyxCallback onyxCallback = new OnyxConfiguration.OnyxCallback() {
        @Override
        public void onConfigured(Onyx onyx) {
            configuredOnyx = onyx;
            if (configuredOnyx != null) {
                OnyxPlugin.getActivity().runOnUiThread(() -> {
                    OnyxPlugin.channel.invokeMethod("onyx_configured", null);
                    Log.i("OnyxPlugin", "onyx configured");
                    startOnyx();
                });
            }
        }
    };

    // #region The callback methods

    /// This should return a flutter callback result
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        try {
            switch (call.method) {
                case "configureOnyx": {
                    configureOnyx(call);
                    result.success("Onyx Configured");
                    break;
                }
                default: {
                    result.notImplemented();
                    break;
                }
            }
        } catch (Exception ex) {
            Log.i("OnyxPlugin", "Exception: " + ex.getMessage(), ex);
            result.error("1", ex.getMessage(), ex);
        }
    }

    // #region The channel event methods.
    /// starts onyx
    private void startOnyx() {
        Log.i("OnyxPlugin", "Onyx starting");
        OnyxPlugin.getActivity().startActivityForResult(
                new Intent(OnyxPlugin.getActivity(), OnyxActivity.class), 1337);
        Log.i("OnyxPlugin", "Onyx started");
    }

    /// configures onyx.
    private void configureOnyx(@NonNull MethodCall call) {
        Log.i("OnyxPlugin", "Onyx Configuration started");
        OnyxConfigurationBuilder onyxConfigurationBuilder = new OnyxConfigurationBuilder()
                .configActivity(OnyxPlugin.getActivity()).licenseKey(call.argument("licenseKey"))
                .successCallback(successCallback)
                .errorCallback(errorCallback)
                .onyxCallback(onyxCallback)
                .returnRawImage(OnyxCallbackHelpers.getBoolValue(call, "returnRawImage"))
                .returnProcessedImage(OnyxCallbackHelpers.getBoolValue(call, "returnProcessedImage"))
                .returnEnhancedImage(OnyxCallbackHelpers.getBoolValue(call, "returnEnhancedImage"))
                .returnSlapImage(OnyxCallbackHelpers.getBoolValue(call, "returnSlapImage"))
                .shouldBinarizeProcessedImage(OnyxCallbackHelpers.getBoolValue(call, "shouldBinarizeProcessedImage"))
                .returnFullFrameImage(OnyxCallbackHelpers.getBoolValue(call, "returnFullFrameImage"))
                .returnWSQ(OnyxCallbackHelpers.getBoolValue(call, "returnWSQ"))
                .cropSize(OnyxCallbackHelpers.getDoubleValue(call, "cropSizeWidth"),
                        OnyxCallbackHelpers.getDoubleValue(call, "cropSizeHeight"))
                .cropFactor(OnyxCallbackHelpers.getFloatValue(call, "cropFactor"))
                .showLoadingSpinner(OnyxCallbackHelpers.getBoolValue(call, "showLoadingSpinner"))
                .useManualCapture(OnyxCallbackHelpers.getBoolValue(call, "useManualCapture"))
                .useOnyxLive(OnyxCallbackHelpers.getBoolValue(call, "useOnyxLive"))
                .computeNfiqMetrics(OnyxCallbackHelpers.getBoolValue(call, "nfiqMetrics"))
                .useFlash(OnyxCallbackHelpers.getBoolValue(call, "useFlash"))
                .uploadMetrics(OnyxCallbackHelpers.getBoolValue(call, "uploadMetrics"))
                .returnOnyxErrorOnLowQuality(OnyxCallbackHelpers.getBoolValue(call, "returnOnyxErrorOnLowQuality"))
                .captureQualityThreshold(OnyxCallbackHelpers.getDoubleValue(call, "captureQualityThreshold"))
                .enableShutterSound(OnyxCallbackHelpers.getBoolValue(call, "shutterSoundEnabled"));

        String fingerprintTemplateTypeText = call.argument("fingerprintTemplateType");
        OnyxConfiguration.FingerprintTemplateType fingerprintTemplateType =
                OnyxConfiguration.FingerprintTemplateType.NONE;
        if (OnyxConfiguration.FingerprintTemplateType.INNOVATRICS.toString()
                .equalsIgnoreCase(fingerprintTemplateTypeText)) {
            fingerprintTemplateType = OnyxConfiguration.FingerprintTemplateType.INNOVATRICS;
        } else if (OnyxConfiguration.FingerprintTemplateType.ISO.toString()
                .equalsIgnoreCase(fingerprintTemplateTypeText)) {
            fingerprintTemplateType = OnyxConfiguration.FingerprintTemplateType.ISO;
        }
        onyxConfigurationBuilder.returnFingerprintTemplate(fingerprintTemplateType);

        String reticleOrientationText = call.argument("reticleOrientation");
        Reticle.Orientation reticleOrientation = Reticle.Orientation.LEFT;
        if (Reticle.Orientation.RIGHT.toString().equalsIgnoreCase(reticleOrientationText)) {
            reticleOrientation = Reticle.Orientation.RIGHT;
        } else if (Reticle.Orientation.THUMB_PORTRAIT.toString().equalsIgnoreCase(reticleOrientationText)) {
            reticleOrientation = Reticle.Orientation.THUMB_PORTRAIT;
        }
        onyxConfigurationBuilder.reticleOrientation(reticleOrientation);

        try {
            onyxConfigurationBuilder.targetPixelsPerInch(OnyxCallbackHelpers.getDoubleValue(call, "targetPixelsPerInch"));
        } catch (Exception ex) {
            // yes. we're assuming the error is due to the targetPPI being null.
            onyxConfigurationBuilder.targetPixelsPerInch(-1.0);
        }
        if (call.argument("fullFrameMaxImageHeight") != null && call.argument("fullFrameMaxImageHeight") != "") {
            try {
                onyxConfigurationBuilder.fullFrameMaxImageHeight(OnyxCallbackHelpers.getFloatValue(call, "fullFrameMaxImageHeight"));
            } catch (Exception e) {
                Log.e("OnyxPlugin", "Exception getting fullFrameMaxImageHeight");
            }
        }

        if (call.argument("subjectId") != null && call.argument("subjectId") != "") {
            onyxConfigurationBuilder.subjectId(call.argument("subjectId"));
        }

        if (call.argument("manualCaptureText") != null && call.argument("manualCaptureText") != "") {
            onyxConfigurationBuilder.manualCaptureText(call.argument("manualCaptureText"));
        }

        onyxConfigurationBuilder.buildOnyxConfiguration();
        Log.i("OnyxPlugin", "Onyx Configuration Built");
    }
    // #endregion
}
