#import "OnyxPlugin.h"
#import <Flutter/Flutter.h>

@implementation OnyxPlugin

static FlutterMethodChannel* channel;
Onyx* _configuredOnyx;
static FlutterViewController * _flutterController;

#pragma mark - flutter controller Property
+(FlutterViewController *)flutterViewController{
    return _flutterController;
}
+(void) setFlutterViewController:(FlutterViewController *) newFlutterViewController{
    _flutterController=newFlutterViewController;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    channel = [FlutterMethodChannel
                methodChannelWithName:@"com.dft.onyx_plugin/methodChannel"
                binaryMessenger:[registrar messenger]];
    OnyxPlugin* instance = [[OnyxPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"configureOnyx" isEqualToString:call.method]) {
      [[self configureOnyx: call] buildOnyxConfiguration];
      result(0);
  } else {
      result(FlutterMethodNotImplemented);
  }
}

#pragma mark - Onyx Callbacks

-(void(^)(Onyx* configuredOnyx))onyxCallback {
    return ^(Onyx* configuredOnyx) {
        NSLog(@"Onyx Callback");
        dispatch_async(dispatch_get_main_queue(), ^{
            _configuredOnyx = configuredOnyx;
            [channel invokeMethod:@"onyx_configured" arguments:nil];
            if(_configuredOnyx != nil && [[self topViewController] isMemberOfClass:[FlutterViewController class]]) {
                [_configuredOnyx capture:_flutterController];
            } else {
                NSLog(@"Unable to start ONYX.");
            }
        });
    };
}

-(void(^)(OnyxResult* onyxResult))onyxSuccessCallback {
    return ^(OnyxResult* onyxResult) {
        NSLog(@"Onyx Success Callback");
        NSMutableDictionary* flutterResults=  [self getOnyxResultFlutterParams:onyxResult];
        [_flutterController.navigationController popToRootViewControllerAnimated:YES];
        dispatch_async(dispatch_get_main_queue(), ^{
                [channel invokeMethod:@"onyx_success" arguments:flutterResults];
        });
        [_flutterController.navigationController popToRootViewControllerAnimated:YES];
    };
}

/*
handles the onyx error callback.
*/
-(void(^)(OnyxError* onyxError)) onyxErrorCallback {
    return ^(OnyxError* onyxError) {
        dispatch_async(dispatch_get_main_queue(), ^{
            NSLog(@"Onyx Error Callback");
            [channel invokeMethod:@"onyx_error"
                        arguments:@{@"errorMessage":  onyxError.errorMessage}
             ];
        });
    };
}


/*
 Configures onyx with the passed in parameters.
 */
-(OnyxConfigurationBuilder*) configureOnyx:(FlutterMethodCall*)call{
    OnyxConfigurationBuilder* onyxConfigBuilder = [[OnyxConfigurationBuilder alloc] init];
    onyxConfigBuilder
        .licenseKey(call.arguments[@"licenseKey"])
        .successCallback([self onyxSuccessCallback])
        .errorCallback([self onyxErrorCallback])
        .onyxCallback([self onyxCallback])
        .returnRawImage([call.arguments[@"returnRawImage"] boolValue])
        .returnProcessedImage([call.arguments[@"returnProcessedImage"] boolValue])
        .returnEnhancedImage([call.arguments[@"returnEnhancedImage"] boolValue])
        .returnSlapImage([call.arguments[@"returnSlapImage"] boolValue])
        .shouldBinarizeProcessedImage([call.arguments[@"shouldBinarizeProcessedImage"] boolValue])
        .returnFullFrameImage([call.arguments[@"returnFullFrameImage"] boolValue])
        .returnWSQ([call.arguments[@"returnWSQ"] boolValue])
        .showLoadingSpinner([call.arguments[@"showLoadingSpinner"] boolValue])
        .useManualCapture([call.arguments[@"useManualCapture"] boolValue])
        .useOnyxLive([call.arguments[@"useOnyxLive"] boolValue])
        .useFlash([call.arguments[@"useFlash"] boolValue])
        .computeNfiqMetrics([call.arguments[@"computeNfiqMetrics"] boolValue])
        .subjectId(call.arguments[@"subjectId"])
        .uploadMetrics([call.arguments[@"uploadMetrics"] boolValue])
        .returnOnyxErrorOnLowQuality([call.arguments[@"returnOnyxErrorOnLowQuality"] boolValue])
        .captureQualityThreshold([call.arguments[@"captureQualityThreshold"] floatValue]);

    FingerprintTemplateType fingerprintTemplateType = NONE;
    if ([@"INNOVATRICS" isEqualToString:call.arguments[@"fingerprintTemplateType"]]) {
        fingerprintTemplateType = INNOVATRICS;
    } else if ([@"ISO" isEqualToString:call.arguments[@"fingerprintTemplateType"]]) {
        fingerprintTemplateType = ISO;
    }
    onyxConfigBuilder.returnFingerprintTemplate(fingerprintTemplateType);

    ReticleOrientation reticleOrientation=LEFT;
    if([@"RIGHT" isEqualToString:call.arguments[@"reticleOrientation"]]) {
        reticleOrientation = RIGHT;
    } else if([@"THUMB_PORTRAIT" isEqualToString:call.arguments[@"reticleOrientation"]]) {
        reticleOrientation = THUMB_PORTRAIT;
    }
    onyxConfigBuilder.reticleOrientation(reticleOrientation);
    // Crop Factor
    if (![call.arguments[@"cropFactor"] isEqualToString:@""]) {
        onyxConfigBuilder.cropFactor([call.arguments[@"cropFactor"] floatValue]);
    }
    // Crop Size
    if (![call.arguments[@"cropSizeHeight"] isEqualToString:@""] && ![call.arguments[@"cropSizeWidth"] isEqualToString:@""]) {
        onyxConfigBuilder.cropSize(CGSizeMake([call.arguments[@"cropSizeWidth"] floatValue], [call.arguments[@"cropSizeHeight"] floatValue]));
    }
    // Full Frame Max Image Height
    if (![call.arguments[@"fullFrameMaxImageHeight"] isEqualToString:@""]) {
        onyxConfigBuilder.fullFrameMaxImageHeight([call.arguments[@"fullFrameMaxImageHeight"] floatValue]);
    }
    // Target Pixels Per Inch
    if (![call.arguments[@"targetPixelsPerInch"] isEqualToString:@""]) {
        onyxConfigBuilder.targetPixelsPerInch([call.arguments[@"targetPixelsPerInch"] floatValue]);
    }
    // Manual Capture Text
    if (![call.arguments[@"manualCaptureText"] isEqualToString:@""]) {
        onyxConfigBuilder.manualCaptureText(call.arguments[@"manualCaptureText"]);
    }
    return onyxConfigBuilder;
}

///Gets the onyx results as a dictionary of params to pass to flutter.
-(NSMutableDictionary*) getOnyxResultFlutterParams:(OnyxResult*) onyxResult{
    NSMutableDictionary *dict = [[NSMutableDictionary alloc]initWithCapacity:8];
    //add images
    if ([onyxResult getRawFingerprintImages] != nil) {
        [dict setObject:[NSMutableArray arrayWithArray:[self getImageByteArrays: [onyxResult getRawFingerprintImages]]] forKey:@"rawFingerprintImages"];
    }
    if ([onyxResult getProcessedFingerprintImages] != nil) {
        [dict setObject:[NSMutableArray arrayWithArray:[self getImageByteArrays: [onyxResult getProcessedFingerprintImages]]] forKey:@"processedFingerprintImages"];
    }
    if ([onyxResult getEnhancedFingerprintImages] != nil) {
        [dict setObject:[NSMutableArray arrayWithArray:[self getImageByteArrays: [onyxResult getEnhancedFingerprintImages]]] forKey:@"enhancedFingerprintImages"];
    }
    // TODO figure out how to do a nil check for this
//    if ([onyxResult getSlapImage] != nil) {
//        [dict setObject:UIImagePNGRepresentation([onyxResult getSlapImage]) forKey:@"slapImage"];
//    }
//    if ([onyxResult getFullFrameImage] != nil) {
//        [dict setObject:UIImagePNGRepresentation([onyxResult getFullFrameImage]) forKey:@"fullFrameImage"];
//    }
    if ([onyxResult getWsqData] != nil) {
        [dict setObject:[NSMutableArray arrayWithArray:[onyxResult getWsqData]] forKey:@"wsqData"];
    }
    //add templates
    [dict setObject:[NSMutableArray arrayWithArray:[self getTemplateFlutterArrays: [onyxResult getFingerprintTemplates]]] forKey:@"iosFingerprintTemplates"];

    //add metrics
    CaptureMetrics* metrics= [onyxResult getMetrics];
    if(metrics != nil){
        [dict setObject:[NSNumber numberWithFloat:[metrics getLivenessConfidence]] forKey:@"livenessConfidence"];
        [dict setObject:[NSNumber numberWithFloat:[metrics getQualityMetric]] forKey:@"qualityMetric"];
        NSMutableArray* nfiqArray=[metrics getNfiqMetrics];
        NSUInteger nfiqCount = [nfiqArray count];
        NSMutableArray *nfiqScores = [[NSMutableArray alloc] initWithCapacity:nfiqCount];
        for(NfiqMetrics* nfiq in nfiqArray){
            if(nfiq != nil){
                [nfiqScores addObject: [NSNumber numberWithInt:[nfiq getNfiqScore]]];
            }
        }
        [dict setObject:[NSMutableArray arrayWithArray:nfiqScores] forKey:@"nfiqScores"];
    }
    return dict;
}

/*
 Converts an array of templates for flutter.
*/
-(NSMutableArray*) getTemplateFlutterArrays:(NSMutableArray*) templateData{
    if(!templateData){
        return [[NSMutableArray alloc] initWithCapacity:0];
    }
    NSUInteger count = [templateData count];
    NSMutableArray *returnArray = [[NSMutableArray alloc] initWithCapacity:count];
    for (NSData* data in templateData) {
        [returnArray addObject:  [data base64EncodedDataWithOptions:NSUTF8StringEncoding]];
    }
    return returnArray;
}

/*
 Converts an array of UIImages to an array of byte arrays.
*/
-(NSMutableArray*) getImageByteArrays:(NSMutableArray*) imageArray{
    if(!imageArray){
        return [[NSMutableArray alloc] initWithCapacity:0];
    }
    NSUInteger count = [imageArray count];
    NSMutableArray *returnArray = [[NSMutableArray alloc] initWithCapacity:count];
    for (UIImage* image in imageArray) {
        [returnArray addObject:  UIImagePNGRepresentation(image)];
    }
    return returnArray;
}

- (UIViewController *)topViewController{
   return [self topViewController:[UIApplication sharedApplication].keyWindow.rootViewController];
 }

 - (UIViewController *)topViewController:(UIViewController *)rootViewController
 {
   if ([rootViewController isKindOfClass:[UINavigationController class]]) {
     UINavigationController *navigationController = (UINavigationController *)rootViewController;
     return [self topViewController:[navigationController.viewControllers lastObject]];
   }
   if ([rootViewController isKindOfClass:[UITabBarController class]]) {
     UITabBarController *tabController = (UITabBarController *)rootViewController;
     return [self topViewController:tabController.selectedViewController];
   }
   if (rootViewController.presentedViewController) {
     return [self topViewController:rootViewController];
   }
   return rootViewController;
 }

@end
